from typing import Any, Literal, Mapping, Optional, Sequence, Union
from warnings import warn

from airflow.models import BaseOperator
from airflow.models.xcom import XCOM_RETURN_KEY

from conveyor.secrets import SecretValue as SecretValue


class ConveyorContainerOperatorV2(BaseOperator):
    def __init__(
        self,
        *,
        instance_type: Optional[str] = None,
        airflow_worker_instance_type: Optional[str] = None,
        validate_docker_image_exists=True,
        image: str = "{{ macros.conveyor.default_image(dag) }}",
        command: Optional[Sequence[str]] = None,
        arguments: Optional[Sequence[str]] = None,
        env_vars: Optional[Mapping[str, Union[str, SecretValue]]] = None,
        aws_role: Optional[str] = None,
        azure_application_client_id: Optional[str] = None,
        instance_life_cycle: Optional[Literal["spot", "on_demand"]] = None,
        disk_size: Optional[int] = None,
        disk_mount_path: str = "/var/data",
        xcom_push: bool = False,
        xcom_key: str = XCOM_RETURN_KEY,
        **kwargs,
    ) -> None: ...


class ConveyorSparkSubmitOperatorV2(BaseOperator):
    def __init__(
        self,
        *,
        application: str = "",
        application_args: Optional[Sequence[Any]] = None,
        conf: Optional[Mapping[str, str]] = None,
        java_class: Optional[str] = None,
        num_executors: Optional[int] = None,
        spark_main_version: Optional[int] = None,
        validate_docker_image_exists=True,
        driver_instance_type: Optional[str] = None,
        executor_instance_type: Optional[str] = None,
        aws_role: Optional[str] = None,
        azure_application_client_id: Optional[str] = None,
        image: str = "{{ macros.conveyor.default_image(dag) }}",
        env_vars: Optional[Mapping[str, Union[str, SecretValue]]] = None,
        instance_life_cycle: Optional[
            Literal["spot", "on-demand", "driver-on-demand-executors-spot"]
        ] = None,
        airflow_worker_instance_type: Optional[str] = None,
        s3_committer: Optional[Literal["file", "magic"]] = None,
        abfs_committer: Optional[Literal["file", "manifest"]] = None,
        executor_disk_size: Optional[int] = None,
        mode: Optional[Literal["local", "cluster", "cluster-v2"]] = None,
        aws_availability_zone: Optional[str] = None,
        verbose: bool = False,
        **kwargs,
    ) -> None:
        if spark_main_version is not None:
            warn(
                "Setting spark main version is deprecated, "
                "the version is automatically detected from the container image",
                DeprecationWarning,
            )



import datetime
from typing import Callable, Iterable, Literal, Mapping, Optional, Sequence

from airflow.sensors.base import BaseSensorOperator


class ConveyorContainerSensor(BaseSensorOperator):
    def __init__(
        self,
        *,
        instance_type: Optional[str] = None,
        instance_life_cycle: Optional[Literal["spot", "on_demand"]] = None,
        image: str = "{{ macros.conveyor.default_image(dag) }}",
        command: Optional[Sequence[str]] = None,
        arguments: Optional[Sequence[str]] = None,
        env_vars: Optional[Mapping[str, str]] = None,
        aws_role: Optional[str] = None,
        azure_application_client_id: Optional[str] = None,
        **kwargs,
    ) -> None: ...


class ConveyorExternalTaskSensor(BaseSensorOperator):
    def __init__(
        self,
        *,
        external_dag_id: str,
        environment: Optional[str] = None,
        external_task_id: Optional[str] = None,
        external_task_ids: Optional[Iterable[str]] = None,
        allowed_states: Optional[Iterable[str]] = None,
        execution_delta: Optional[datetime.timedelta] = None,
        execution_date_fn: Optional[Callable] = None,
        instance_life_cycle: Optional[Literal["spot", "on_demand"]] = None,
        **kwargs,
    ) -> None: ...



from typing import Optional


class SecretValue:
    pass


class AWSSecretsManagerValue(SecretValue):
    def __init__(self, name: str, path: Optional[str] = None) -> None: ...


class AWSParameterStoreValue(SecretValue):
    def __init__(self, name: str, path: Optional[str] = None) -> None: ...


class AzureKeyVaultValue(SecretValue):
    def __init__(self, name: str, vault: str, vault_type: str = "secret") -> None: ...



from typing import Literal, Mapping, Optional, Sequence, Tuple, Union

from airflow import DAG
from airflow.operators.empty import EmptyOperator
from airflow.utils.log.logging_mixin import LoggingMixin
from airflow.utils.task_group import TaskGroup

from conveyor.secrets import SecretValue


class ConveyorDbtTaskFactory(LoggingMixin):
    def __init__(
        self,
        *,
        manifest_file: str = "manifest.json",
        task_name_prefix: Optional[str] = None,
        task_name_suffix: Optional[str] = None,
        task_cmd: Sequence[str] = (),
        task_arguments: Sequence[str] = (
            "--no-use-colors",
            "{command}",
            "--target",
            "{{ macros.conveyor.env() }}",
            "--profiles-dir",
            "./..",
            "--select",
            "{model}",
        ),
        task_instance_type: str = "mx.micro",
        airflow_worker_instance_type: Optional[str] = None,
        task_instance_life_cycle: Optional[Literal["spot", "on-demand"]] = None,
        task_aws_role: Optional[str] = None,
        task_azure_application_client_id: Optional[str] = None,
        task_env_vars: Optional[Mapping[str, Union[str, SecretValue]]] = None,
        start_task_name_override: Optional[str] = None,
        end_task_name_override: Optional[str] = None,
    ) -> None: ...

    def add_tasks_to_dag(
        self,
        dag: DAG,
        *,
        tags: Sequence[str] = (),
        any_tag: bool = True,
        test_tasks: bool = True,
    ) -> Tuple[EmptyOperator, EmptyOperator]: ...

    def add_tasks_to_dag_v2(
        self,
        dag: DAG,
        *,
        select: Optional[str] = None,
        exclude: Optional[str] = None,
        test_tasks: bool = True,
    ) -> Tuple[EmptyOperator, EmptyOperator]: ...

    def add_tasks_to_task_group(
        self,
        dag: DAG,
        *,
        task_group_name: str = "dbt_run",
        test_task_group_name: str = "dbt_test",
        tags: Sequence[str] = (),
        any_tag: bool = True,
    ) -> Tuple[TaskGroup, TaskGroup]: ...

    def add_tasks_to_task_group_v2(
        self,
        dag: DAG,
        *,
        task_group_name: str = "dbt_run",
        test_task_group_name: str = "dbt_test",
        select: Optional[str] = None,
        exclude: Optional[str] = None,
    ) -> Tuple[TaskGroup, TaskGroup]: ...
